package com.zy.bms.config.mqtt;

import com.alibaba.fastjson.JSON;
import com.zy.bms.redis.DeviceManager;
import com.zy.bms.service.DeviceLocationService;
import com.zy.bms.service.DeviceService;
import com.zy.bms.service.SetToDeviceService;
import com.zy.bms.utils.CodeGenerator;
import com.zy.bms.utils.GaoDeApiUtil;
import com.zy.bms.websocket.WebSocketServer;
import com.zy.bms.model.CallRecords;
import com.zy.bms.model.Device;
import com.zy.bms.model.DeviceLocation;
import com.zy.bms.model.MqttMsgDto;
import com.zy.bms.service.CallRecordsService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * MQTT 消息返回处理类
 *
 * @author yang xiao kun
 * create on 2021/1/19
 */
@Component
public class MqttCallbackHandler {

    @Resource
    private DeviceService deviceService;
    @Resource
    private SetToDeviceService setToDeviceService;
    @Resource
    private DeviceLocationService deviceLocationService;
    @Resource
    private GaoDeApiUtil gaoDeApiService;
    @Resource
    private DeviceManager deviceRedisService;
    @Resource
    private WebSocketServer webSocketServer;
    @Resource
    private CallRecordsService callRecordsService;

    /**
     * 处理消息
     *
     * @param topic   主题
     * @param payload 消息内容
     */
    void handle(String topic, String payload) {
        System.out.println(payload);
        System.out.println("------------------------------------------------------");
        if (topic.contains("/")) {
            webSocketServer.massMessage(topic.split("/")[1], payload);
        }
        //消息返回JSON转Object
        MqttMsgDto msgObject = JSON.parseObject(payload, MqttMsgDto.class);
        //根据报文标识代码 M 处理消息
        switch (msgObject.getM()) {
//            case "10"://设备端注册报文
//                registerDevice(msgObject);
//                break;
//            case "12"://设备注册完成
//                setToDeviceService.registerOK(msgObject.getData().get("deviceid").toString());
//                break;
            case "20":// 设备位置信息上报
                LocationHandler(msgObject);
                break;
            case "40"://设备上传通话记录
                CallRecordsHandler(msgObject);
                break;
        }
    }

    /**
     * 注册设备
     */
    private void registerDevice(MqttMsgDto obj) {
        String num = obj.getData().get("num").toString();
        //查询数据库中是否有此设备
        Device device = deviceService.getByNum(num);
        if (device == null) {
            device = new Device(num, CodeGenerator.generateClientId(), CodeGenerator.generateShortUuid());
            //没有该设备,保存至数据库
            deviceService.save(device);
        }
        //发送消息
        setToDeviceService.returnCodeToDevice(device.getNum(), device.getClientId(), device.getPassword());
    }

    /**
     * 处理上传位置
     */
    private void LocationHandler(MqttMsgDto obj) {
        DeviceLocation location = new DeviceLocation(obj);
        // 逆地理位置解析
        location.setSite(gaoDeApiService.regeo(location.getLonGcj(), location.getLatGcj()));
        // 保存位置信息--历史记录
        deviceLocationService.save(location);
        // 将最新位置信息保存至redis中
        deviceRedisService.saveLocation(location);
    }

    /**
     * 设备上传通话记录
     */
    private void CallRecordsHandler(MqttMsgDto obj) {
        CallRecords callRecords = new CallRecords();
        callRecords.setDeviceId(obj.getR().split("/")[1]);
        callRecords.setType((int) obj.getData().get("type"));
        callRecords.setKey((String) obj.getData().get("key"));
        callRecords.setPhoneNum((String) obj.getData().get("phnoenum"));
        callRecords.setTalkTime((int) obj.getData().get("talktime"));
        callRecords.setEndTime((String) obj.getData().get("endtime"));
        callRecordsService.save(callRecords);
    }

}
